@@ -20,16 +20,13 @@ linarpt <- function(Y, id = NULL, weight = NULL, sort = NULL,
 
    # check 'p'
    p <- percentage
-   if(!is.numeric(p) || length(p) != 1 || p[1] < 0 || p[1] > 100) {
-          stop("'percentage' must be a numeric value in [0,100]")
-      } else p <- percentage[1]
+   if(length(p) != 1 |  any(!is.numeric(p) | p < 0 | p > 100)) {
+          stop("'percentage' must be a numeric value in [0, 100]")  }
 
    # check 'order_quant'
-
    oq <- order_quant
-   if(!is.numeric(oq) || length(oq) != 1 || oq[1] < 0 || oq[1] > 100) {
-          stop("'order_quant' must be a numeric value in [0,100]")
-      } else order_quant <- order_quant[1]
+   if(length(oq) != 1 | any(!is.numeric(oq) | oq < 0 | oq > 100)) {
+          stop("'order_quant' must be a numeric value in [0, 100]")  }
 
    if(!is.null(dataset)) {
        dataset <- data.table(dataset)
@@ -85,7 +82,7 @@ linarpt <- function(Y, id = NULL, weight = NULL, sort = NULL,
                       paste(names(period)[duplicated(names(period))], collapse = ","))
        if (nrow(period) != n) stop("'period' must be the same length as 'Y'")
        if(any(is.na(period))) stop("'period' has unknown values")
-       period[, (names(period)):=lapply(.SD, as.character)]  
+       period[, (names(period)):=lapply(.SD, as.character)]
    }   
       
    # id
