@@ -1,9 +1,11 @@
+
 vardchanges <- function(Y, H, PSU, w_final, id,
                      Dom = NULL, Z=NULL, 
                      country, periods,
                      dataset = NULL,
                      period1, period2,
                      linratio = FALSE,
+                     percentratio = FALSE,
                      use.estVar = FALSE,
                      confidence=0.95,
                      change_type="absolute") {
@@ -12,6 +14,7 @@ vardchanges <- function(Y, H, PSU, w_final, id,
 
   if (!change_type %in% c("absolute", "relative")) stop("'change_type' must be 'absolute' or 'relative'")
   if (length(linratio) != 1 | !any(is.logical(linratio))) stop("'linratio' must be the logical value")
+  if (length(percentratio) != 1 | !any(is.logical(percentratio))) stop("'percentratio' must be the logical value") 
   if (length(use.estVar) != 1 | !any(is.logical(use.estVar))) stop("'use.estVar' must be the logical value")
   if(length(confidence) != 1 | any(!is.numeric(confidence) |  confidence < 0 | confidence > 1)) {
           stop("'confidence' must be a numeric value in [0,1]")  }
@@ -101,7 +104,7 @@ vardchanges <- function(Y, H, PSU, w_final, id,
   periods <- data.table(periods, check.names=TRUE)
   if (any(is.na(periods))) stop("'periods' has unknown values")
   if (nrow(periods) != n) stop("'periods' length must be equal with 'Y' row count")
-  if (ncol(country) != 1) stop("'periods' must be 1 column")
+  if (ncol(periods) != 1) stop("'periods' must be 1 column")
 
   # Dom
   if (!is.null(Dom)) {
@@ -132,26 +135,33 @@ vardchanges <- function(Y, H, PSU, w_final, id,
    if (any(is.na(period1))) stop("'period1' has unknown values")
    periodss <- copy(periods)
    periodss[, periodss:=1]
-   if (any(is.na(merge(period1, periodss, all.x=TRUE, by=names(periods)))))
+
+   if (any(is.na(merge(period1, periodss, all.x=TRUE,
+                       by=names(periods), allow.cartesian=TRUE))))
               stop("'period1' row must be exist in 'periods'")
 
+
    # period2
    period2 <- data.table(period2, check.names=TRUE)
    if (ncol(period2) != 1) stop("'period2' must be 1 column")
    setnames(period2, names(period2), names(periods))
    if (any(is.na(period2))) stop("'period2' has unknown values")
-   if (any(is.na(merge(period2, periodss, all.x=TRUE, by=names(periods)))))
-              stop("'period2' row must be exist in 'periods'")
+   if (any(is.na(merge(period2, periodss, all.x=TRUE,
+                       by=names(periods), allow.cartesian=TRUE))))
+             stop("'period2' row must be exist in 'periods'")
 
-   data <- vardcros(Y=Y, H=H, PSU=PSU, w_final=w_final,
+   datas <- vardcros(Y=Y, H=H, PSU=PSU, w_final=w_final,
                     id=id, Dom=Dom, Z=Z, country=country,
                     periods=periods, dataset=NULL,
-                    linratio=linratio, use.estVar=use.estVar,
+                    linratio=linratio, 
+                    percentratio=percentratio,
+                    use.estVar = use.estVar,
                     household_level_max=TRUE,
                     withperiod=TRUE, netchanges=TRUE, 
                     confidence=confidence)
 
-  crossectional_results <- data$results
+  crossectional_results <- datas$results
+
   if (!is.null(Dom)) { 
         Y1 <- names(domain(Y, Dom))
         if (!is.null(Z)) Z1 <- names(domain(Z, Dom))
@@ -173,39 +183,85 @@ vardchanges <- function(Y, H, PSU, w_final, id,
   num1num1 <- den1den1 <- den1 <- num2num2 <- NULL
   den2den2 <- den2 <- num1den1 <- num1num2 <- NULL
   num2 <- num1den2 <- den1num2 <- den1den2 <- num2den2 <- NULL
-  C22 <- C12 <- num1_1 <-  den1_1 <- num1den1 <- NULL
-  C11 <- den1den1 <- C13 <- num1_2 <- C14 <- den1_2 <- NULL
-  C23 <- C24 <- C33 <- C34 <- C44 <- estim <- estim_1 <- NULL
-  estim_2 <- grad1_1 <- grad1_2 <- CI_upper <- grad2_1 <- NULL
-  rot <- grad2_2 <- se <-  CI_lower <-  NULL
+  num1_1 <- den1_1 <- num1den1 <- den1den1 <- num1_2 <- NULL
+  den1_2 <- estim <- estim_1 <- estim_2 <- NULL
+  grad1_1 <- grad1_2 <- CI_upper <- grad2_1 <- NULL
+  ids_nr <- rot <- grad2_2 <- se <-  CI_lower <-  NULL
+  valueY1_1 <- valueZ1_1 <- valueY1_2 <- valueZ1_2 <- NULL
+  id_nams <- nams <- ids_nr <- NULL
 
-  var_grad <- data$var_grad
+  var_grad <- datas$var_grad
+  cros_var_grad <- copy(var_grad)
   period1[, ind:=.I]
   period2[, ind:=.I]
-  var_grad1 <- merge(period1, var_grad, all.x=TRUE, by=per)
-  var_grad2 <- merge(period2, var_grad, all.x=TRUE, by=per)
+  var_grad1 <- merge(period1, var_grad, all.x=TRUE,
+                              by=per, allow.cartesian=TRUE)
+  var_grad2 <- merge(period2, var_grad, all.x=TRUE,
+                              by=per, allow.cartesian=TRUE)
   
   sarc <- c("ind", "namesY", "namesZ", country, Dom)
   sarc <- names(var_grad1)[(names(var_grad1) %in% sarc)]
   sar <- names(var_grad1)[!(names(var_grad1) %in% sarc)]
   setnames(var_grad1, sar, paste0(sar, "_1"))
   setnames(var_grad2, sar, paste0(sar, "_2"))
-
   var_grad <- merge(var_grad1, var_grad2, all=TRUE, by=sarc)
-  
-  var_grad1 <- var_grad2 <- NULL
+  var_grad[, ids_nr:=1:.N]
+
+  if (change_type=="relative"){
+        if (linratio==FALSE){
+             var_grad[, grad1_1:=-valueY1_2*valueZ1_1/(valueZ1_2*(valueY1_1)^2)]
+             var_grad[, grad1_2:=valueY1_2/(valueZ1_2*valueY1_1)]
+             var_grad[, grad2_1:=valueZ1_1/(valueZ1_2*valueY1_1)]
+             var_grad[, grad1_1:=-valueY1_2*valueZ1_1/((valueZ1_2)^2*valueY1_1)]
+          } else {
+             var_grad[, grad1_1:=-valueY1_2/(valueY1_1)^2]
+             var_grad[, grad1_2:=1/valueY1_1] }
+     } else {
+        if (!is.null(var_grad$grad1_1)){
+                var_grad[, grad1_1:=-grad1_1]
+                var_grad[, grad2_1:=-grad2_1] 
+           } else {var_grad[, grad1_1:=-1]
+                   var_grad[, grad1_2:=1] }}
+
+  var_grad11 <- copy(var_grad)
+  var_grad12 <- copy(var_grad)
+  var_grad11[, (c("grad", "cros_var", "id_nams", "nams")):=list(grad1_1, num1_1, 1, "num1")]
+  var_grad12[, (c("grad", "cros_var", "nams")):=list(grad1_2, num1_2, "num2")]
+  var_grad12[, id_nams:=2+as.numeric(!is.null(var_grad$grad2_1))]
+
+  var_grad21 <- var_grad22 <- NULL
+  if (!is.null(var_grad$grad2_1)) {
+        var_grad21 <- copy(var_grad)
+        var_grad22 <- copy(var_grad)
+        var_grad21[, (c("grad", "cros_var", "id_nams", "nams")):=list(grad2_1, den1_1, 2, "den1")]
+        var_grad22[, (c("grad", "cros_var", "id_nams", "nams")):=list(grad2_2, den1_2, 4, "den2")]
+   }
+  var_gradn <- rbindlist(list(var_grad11, var_grad12,
+                               var_grad21, var_grad22), fill=TRUE)
+
+  sarc2 <- c(paste0(names(periods), "_", c(1, 2)), sarc)
+  var_gradn <- var_gradn[,c(sarc2, "ids_nr", "id_nams", 
+                            "nams", "grad", "cros_var"), with=FALSE]
+
+  var_grad11 <- var_grad12 <- NULL
+  var_grad21 <- var_grad22 <- NULL
+
+
+
+  data <- datas$data_net_changes
 
-  data <- data$data_net_changes
   data[, rot:=1]
-  data1 <- merge(period1, data, all.x=TRUE, by=per)
-  data2 <- merge(period2, data, all.x=TRUE, by=per)
+  data1 <- merge(period1, data, all.x=TRUE,
+                    by=per, allow.cartesian=TRUE)
+  data2 <- merge(period2, data, all.x=TRUE, 
+                    by=per, allow.cartesian=TRUE)
 
   sard <- names(data)[!(names(data) %in% sarp)]
   setnames(data1, sard, paste0(sard, "_1"))
   setnames(data2, sard, paste0(sard, "_2"))
 
   data <- merge(data1, data2, all=TRUE, by=c("ind", sarp))
-
+  datas <- NULL
   data1 <- data2 <- NULL
 
   recode.NA <- function(DT, cols = seq_len(ncol(DT))) {
@@ -225,9 +281,10 @@ vardchanges <- function(Y, H, PSU, w_final, id,
   fit <- lapply(1:length(Y1), function(i) {
        fitd <- lapply(split(data, data[["ind"]]), function(data1) {
             fits <- lapply(split(data1, data1[[country]]), function(DT3c) {
+
                       y1 <- paste0(Y1[i], "_1")
                       y2 <- paste0(Y1[i], "_2")
-                      if (!is.null(namesZ)) { 
+                      if (!is.null(namesZ) & !linratio) { 
                                     z1 <- paste0(",", Z1[i], "_1") 
 	                              z2 <- paste0(",", Z1[i], "_2") 
                                } else z1 <- z2 <- ""
@@ -243,33 +300,28 @@ vardchanges <- function(Y, H, PSU, w_final, id,
                       
                       if (use.estVar) { res <- data.table(estVar(res))
                                   } else res <- data.table(lm(funkc, data=DT3c)$res)
-                      if (!is.null(namesZ)) { 
+                      if (!is.null(namesZ) & !linratio) { 
                                    setnames(res, names(res), c("num1", "den1", "num2", "den2"))
                                    res[, nameZs:=Z1[i]]
                             } else setnames(res, names(res), c("num1", "num2"))
-
-                      Zvn <- as.integer(!is.null(namesZ))
+                      nosv <- c("num1", "den1", "num2", "den2")
+                      nosv <- names(res)[names(res) %in% nosv]
+                      Zvn <- as.integer(!is.null(namesZ) & !linratio)
                       res[, nameYs:=Y1[i]]
 
-                      if (use.estVar) { 
-                           res[, num1num1:=res[["num1"]][1]]
-                           res[, num2num2:=res[["num2"]][2+Zvn]]
-                           res[, num1num2:=res[["num1"]][2+Zvn]]
-                           if (!is.null(namesZ)) {
-                                 res[, den1den1:=res[["den1"]][1+Zvn]]
-                                 res[, den2den2:=res[["den2"]][3+Zvn]]
-                                 res[, num1den1:=res[["num1"]][1+Zvn]]
-                                 res[, num1den2:=res[["num1"]][3+Zvn]]
+                      keynames <- c(country, "ind", paste0(per, "_1"),
+                                    paste0(per, "_2"), "nameYs", "nameZs")
+                      keynames <- keynames[keynames %in% c(names(DT3c), names(res))]
 
-                                 res[, den1num2:=res[["den1"]][2+Zvn]]
-                                 res[, den1den2:=res[["den1"]][3+Zvn]]
-                                 res[, num2den2:=res[["num2"]][3+Zvn]] }
-                           res <- data.table(res[1], DT3c[1])
+                      if (use.estVar) { 
+                            res <- data.table(id_nams=1:nrow(res), nams=nosv, res, DT3c[1])
                         } else {
                             res[, num1num1:=num1 * num1]
                             res[, num2num2:=num2 * num2]
                             res[, num1num2:=num1 * num2]
-                            if (!is.null(namesZ)) {
+                            res[, id_nams:=0]
+                            res[, nams:=""]
+                            if (!is.null(namesZ) & !linratio) {
                                   res[, den1den1:=den1 * den1]
                                   res[, den2den2:=den2 * den2]
                                   res[, num1den1:=num1 * den1]
@@ -277,18 +329,50 @@ vardchanges <- function(Y, H, PSU, w_final, id,
                                   res[, den1num2:=den1 * num2]
                                   res[, den1den2:=den1 * den2]
                                   res[, num2den2:=num2 * den2] }
-                            res <- data.table(res, DT3c)}
-                      varsp <- c("num1num1", "den1den1",
-                                 "num2num2", "den2den2",
-                                 "num1den1", "num1num2",
-                                 "num1den2", "den1num2",
-                                 "den1den2", "num2den2")
-                      varsp <- varsp[varsp %in% names(res)]
-                      keynames <- c(country, "ind", paste0(per, "_1"),
-                                   paste0(per, "_2"), "nameYs", "nameZs")
-                      keynames <- keynames[keynames %in% names(res)]
-                      fits <- res[, lapply(.SD, sum), keyby=keynames, .SDcols=varsp]
-                      return(fits)
+                            res <- data.table(res, DT3c)
+                            varsp <- c("num1num1", "den1den1",
+                                       "num2num2", "den2den2",
+                                       "num1den1", "num1num2",
+                                       "num1den2", "den1num2",
+                                       "den1den2", "num2den2")
+                            varsp <- varsp[varsp %in% names(res)]
+                            fits <- res[, lapply(.SD, sum), keyby=c(keynames,
+                                                       "id_nams", "nams"),
+                                                       .SDcols=varsp]
+                            fits1 <- copy(fits)
+                            fits1[, (c("id_nams", "nams")):=list(1, "num1")]
+                            setnames(fits1, (c("num1num1", "num1num2")), c("num1", "num2"))
+
+                            fits2 <- copy(fits)
+                            fits2[, id_nams:=2+as.numeric(!is.null(fits$den2den2))]
+                            fits2[, nams:="num2"]
+                            setnames(fits2, c("num1num2", "num2num2"), c("num1", "num2"))
+
+                            fits3 <- fits4 <- NULL
+                            if (!is.null(fits$den2den2)){
+                                 setnames(fits1, c("num1den1", "num1den2"), c("den1", "den2"))
+                                 setnames(fits2, c("den1num2", "num2den2"), c("den1", "den2"))
+
+                                 fits3 <- copy(fits)
+                                 fits3[, (c("id_nams", "nams")):=list(2, "den1")]
+                                 setnames(fits3, c("num1den1", "den1num2",
+                                                   "den1den1", "den1den2"),
+                                                 c("num1", "num2", "den1", "den2"))
+  
+                                 fits4 <- copy(fits)
+                                 fits4[, (c("id_nams", "nams")):=list(4, "den2")]
+                                 setnames(fits4, c("num1den2", "num2den2",
+                                                   "den1den2", "den2den2"),
+                                                 c("num1", "num2", "den1", "den2"))
+                               }
+                            res <- rbindlist(list(fits1, fits2, fits3, fits4), fill=TRUE)
+                            fits <- fits1 <- fits2 <- fits3 <- fits4 <- NULL
+                        }
+                       fits <- res[, lapply(.SD, sum),
+                                      keyby=c(keynames,
+                                      "id_nams", "nams"),
+                                      .SDcols=nosv]
+                    return(fits)
                 })
             rbindlist(fits)
          })
@@ -299,65 +383,81 @@ vardchanges <- function(Y, H, PSU, w_final, id,
    set(res, j=country, value=as.character(res[[country]]))
 
    if (!is.null(Dom)) {
-          var_grad[, paste0(Dom, "_ss"):=lapply(Dom, function(x) paste0(x,".", get(x)))]
-          var_grad[, nameYs:=Reduce(function(x, y)
+          var_gradn[, paste0(Dom, "_ss"):=lapply(Dom, function(x) paste0(x,".", get(x)))]
+          var_gradn[, nameYs:=Reduce(function(x, y)
                                       paste(x, y, sep = "__"), .SD),
                                      .SDcols=c("namesY", paste0(Dom, "_ss"))]
-          if (!is.null(namesZ)) { var_grad[, nameZs:=Reduce(function(x, y)
+          if (!is.null(namesZ)) { var_gradn[, nameZs:=Reduce(function(x, y)
                                                                paste(x, y, sep = "__"), .SD),
                                                              .SDcols=c("namesZ", paste0(Dom, "_ss"))]
                                 }
-       } else { var_grad[, nameYs:=namesY]
-                if (!is.null(namesZ)) var_grad[, nameZs:=namesZ]}
+       } else { var_gradn[, nameYs:=namesY]
+                if (!is.null(namesZ)) var_gradn[, nameZs:=namesZ]}
 
    nameYZ <- c("nameYs", "nameZs")
    nameYZ <- nameYZ[nameYZ %in% names(res)]
 
-   sars <- c(country, "ind", paste0(per, "_1"), paste0(per, "_2"), nameYZ)
-   data <- merge(res, var_grad, all=TRUE, by=sars)
-   res <- fit <- var_grad <- NULL
-   data[, nameYs:=NULL]
+   sars <- c(country, "ind", paste0(per, "_1"), 
+             paste0(per, "_2"), nameYZ)
 
-   data[, C11:=num1_1]
-   data[, C33:=num1_2]
-   data[, C13:=sqrt(num1_1*num1_2/(num1num1*num2num2))*num1num2]
+   data <- merge(res, var_gradn, all=TRUE, by=c(sars, "id_nams", "nams"))
+   res <- fit <- var_gradn <- NULL
 
-   if (!is.null(data$namesZ)) {
-          data[, nameZs:=NULL]   
-          data[, C22:=den1_1]
-          data[, C44:=den1_2]
-          data[, C12:=sqrt(num1_1*den1_1/(num1num1*den1den1))*num1den1]
-          data[, C14:=sqrt(num1_1*den1_2/(num1num1*den2den2))*num1den2]
-          data[, C23:=sqrt(den1_1*num1_2/(den1den1*num2num2))*den1num2]
-          data[, C24:=sqrt(den1_1*den1_2/(den1den1*den2den2))*den1den2]
-          data[, C34:=sqrt(num1_2*den1_2/(num2num2*den2den2))*num2den2]
-        }
+   rmax <- max(data[, .N, by="ids_nr"][["ids_nr"]])
 
-   data[, estim:=estim_1 - estim_2]
-   if (!is.null(namesZ)) {
-         data[, var:= (grad1_1 * grad1_1 * C11) +  
-                      (grad1_2 * grad1_2 * C22) + 
-                      (grad2_1 * grad2_1 * C33) + 
-                      (grad2_2 * grad2_2 * C44) + 
-                   2*((grad1_1 * grad1_2 * C12) +
-                      (grad1_1 * grad2_1 * C13) +
-                      (grad1_1 * grad2_2 * C14) +
-                      (grad1_2 * grad2_1 * C23) +
-                      (grad1_2 * grad2_2 * C24) +
-                      (grad2_1 * grad2_2 * C34))]
-       } else data[, var:=num1_1 + num1_2 - 2 * C13]
+   nosv <- c("num1", "den1", "num2", "den2")
+   nosv <- names(data)[names(data) %in% nosv]
 
-   data[, se:=sqrt(var)]
+   dat <- lapply(1:rmax, function(i) {
+             res <- data[ids_nr==i]
+             res1 <- as.matrix(res[, nosv, with=FALSE])
+             rhod <- diag(sqrt(1/diag(res1)), length(nosv), length(nosv))
+             rhod <- data.table((t(rhod) %*% res1) %*% rhod)
+             setnames(rhod, names(rhod), paste0("rho_", nosv))
+             dmatr <- diag(sqrt(res[["cros_var"]]/diag(res1)),
+                           length(nosv), length(nosv))
+             var_tau <- data.table((t(dmatr) %*% res1) %*% dmatr)
+             dmatr <- data.table(dmatr)
+             setnames(dmatr, names(dmatr), paste0("d_", nosv))
+             setnames(var_tau, names(var_tau), paste0("var_tau_", nosv))
+             res <- data.table(res, rhod, dmatr, var_tau)
+
+             var_t <- (t(res[["grad"]]) %*% as.matrix(var_tau)) %*% res[["grad"]]
+             var_grads <- var_grad[ids_nr==1]
+
+             if (change_type=="absolute") {
+                          var_grads[, estim:=estim_2 - estim_1]
+                     } else var_grads[, estim:=estim_2/estim_1]
+
+             var_grads[, var:=var_t[,1]]
+             list(matricas=res, data=var_grads) })
+
+   matricas <- rbindlist(lapply(dat, function(x) x[[1]]))              
+   datas <- rbindlist(lapply(dat, function(x) x[[2]]))
+
+   datas[var>=0, se:=sqrt(var)]
    tsad <- qnorm(0.5*(1+confidence))
-   data[, CI_lower:=estim - tsad*se]
-   data[, CI_upper:=estim + tsad*se]
+   datas[, CI_lower:=estim - tsad*se]
+   datas[, CI_upper:=estim + tsad*se]
+
+   sarc2 <- c(sarc2, "nams")
+   sarc2 <- sarc2[!(sarc2 %in% "ind")]
+
+   rho_matrix <- matricas[, c(sarc2, paste0("rho_", nosv)), with=FALSE]
+   var_tau <- matricas[, c(sarc2, paste0("var_tau_", nosv)), with=FALSE]
+   grad_var <- matricas[, c(sarc2, "grad", "cros_var"), with=FALSE]
+
    namesYZ <- c("namesY", "namesZ")
-   namesYZ <- namesYZ[namesYZ %in% names(data)]
+   namesYZ <- names(datas)[(names(datas) %in% namesYZ)]
 
-   changes_results <- data[, c(country, paste0(per,"_1"),
-                               paste0(per,"_2"), Dom, namesYZ, 
-                               "estim_1",  "estim_2", "estim", 
-                               "var", "se", "CI_lower", "CI_upper"), with=FALSE]
+   changes_results <- datas[, c(country, paste0(per,"_", c(1, 2)), Dom,
+                                namesYZ, "estim_1",  "estim_2", "estim", 
+                                "var", "se", "CI_lower", "CI_upper"), with=FALSE]
 
- list(crossectional_results=crossectional_results, changes_results=changes_results)
-}
\ No newline at end of file
+   list(crossectional_results=crossectional_results,
+        crossectional_var_grad=cros_var_grad,
+        grad_var=grad_var,
+        rho=rho_matrix,
+        var_tau=var_tau,
+        changes_results=changes_results)
+ }
\ No newline at end of file
